<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/OSL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author Ship-Quik SL <info@ship-quik.com>
 * @copyright Since 2022 Ship-Quik SL
 * @license   https://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 */
include dirname(__FILE__) . '/../../../config/config.inc.php';
include dirname(__FILE__) . '/../../../init.php';
require dirname(__FILE__) . '/../vendor/autoload.php';

use Order;
use ShipQuik\PrestaShop\ShipQuik_BD;
use ShipQuik\Shared\ShipQuik_OrdersPaid;

$module_name = 'ship_quik';

$token = pSQL(Tools::encrypt($module_name . '/ajax_check_orders.php'));
$token_url = pSQL(Tools::getValue('token'));

if ($token != $token_url || !Module::isInstalled($module_name)) {
    exit('Error al ejecutar el ajax');
}

$module = Module::getInstanceByName($module_name);

if ($module->active) {
    $clientId = $module->config->getIdCliente();
    $shipquik_selected_orders = pSQL(Tools::getValue('shipquik_selected_orders'));

    $response = [];

    $result = false;

    if ($shipquik_selected_orders) {
        $references = [];

        foreach (explode(',', $shipquik_selected_orders) as $order_id) {
            $order = new Order((int) $order_id);

            $result = ShipQuik_BD::getDatabyOrderId($order_id);

            if ($result) {
                $orderData = ShipQuik_BD::getDatabyOrder($order);
                $shipmentInfo = json_decode(str_replace('&quot;', '"', $result['shipment_data']), true, 99);
                if (isset($shipmentInfo['data']['reference'])) {
                    $references[] = $shipmentInfo['data']['reference'];
                }
            }
        }

        $ordersPaid = new ShipQuik_OrdersPaid($module, $clientId, $references);
        $result = $ordersPaid->send();

        if ($result) {
            foreach ($result['data'] as $element) {
                $orderData = ShipQuik_BD::getDatabyReference($element['reference']);
                $order = new Order((int) $orderData['id_order']);

                if ($element['paid']) {
                    ShipQuik_BD::setPaidStatus($order, '1');
                }
            }
            $response = ['status' => 200, 'message' => $result];
        } else {
            $response = ['status' => 200, 'error' => 'Error al procesar la consulta.'];
        }
    } else {
        $response = ['status' => 200, 'error' => 'Error al procesar la consulta.'];
    }

    echo json_encode($response);
}
