<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/OSL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author Ship-Quik SL <info@ship-quik.com>
 * @copyright Since 2022 Ship-Quik SL
 * @license   https://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 */
include dirname(__FILE__) . '/../../../config/config.inc.php';
include dirname(__FILE__) . '/../../../init.php';
require dirname(__FILE__) . '/../vendor/autoload.php';

use ShipQuik\PrestaShop\ShipQuik_BD;
use ShipQuik\PrestaShop\ShipQuik_Shipments;
use ShipQuik\Shared\ShipQuik_Supplier;

$module_name = 'ship_quik';

$token = pSQL(Tools::encrypt($module_name . '/ajax_repackaging_totals.php'));
$token_url = pSQL(Tools::getValue('token'));

if ($token != $token_url || !Module::isInstalled($module_name)) {
    exit('Error al ejecutar el ajax');
}

$module = Module::getInstanceByName($module_name);

if ($module->active) {
    $order_id = pSQL(Tools::getValue('order_id'));
    $boxes = Tools::getValue('data');

    $order = new Order((int) $order_id);
    $orderData = ShipQuik_BD::getDatabyOrder($order);
    $shipment = json_decode($orderData['total_payment'], true);

    $supplierInfo = new ShipQuik_Supplier($module, $boxes, new \Address($order->id_address_delivery));

    $suppliers = [
        'name' => $orderData['supplier_data']['supplier'],
        'services' => [$orderData['supplier_data']['service']],
    ];
    $supplierInfo = $supplierInfo->send($suppliers, true);

    $supplierInfo = (isset($supplierInfo[0]) ? $supplierInfo[0] : []);

    $price = 0;
    if (!empty($supplierInfo)) {
        $price = $supplierInfo['salesPrice'];
    }

    $result = new ShipQuik_Shipments($module, $order_id, $boxes, false, $price);

    if (!empty($result->data)) {
        $result->data['order_id'] = $order_id;
        $response = ['status' => 200, 'totals' => $result->data];
    } else {
        $response = ['status' => 200, 'message' => 'Ha habido un problema para solicitar el pedido.'];
    }

    echo json_encode($response);
}
