<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/OSL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author Ship-Quik SL <info@ship-quik.com>
 * @copyright Since 2022 Ship-Quik SL
 * @license   https://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 */
include dirname(__FILE__) . '/../../../config/config.inc.php';
include dirname(__FILE__) . '/../../../init.php';
require dirname(__FILE__) . '/../vendor/autoload.php';

use Order;
use ShipQuik\PrestaShop\ShipQuik_BD;
use ShipQuik\PrestaShop\ShipQuik_Shipments;

$module_name = 'ship_quik';

$token = pSQL(Tools::encrypt($module_name . '/ajax_send_order.php'));
$token_url = pSQL(Tools::getValue('token'));

if ($token != $token_url || !Module::isInstalled($module_name)) {
    exit('Error al ejecutar el ajax');
}

$module = Module::getInstanceByName($module_name);

if ($module->active) {
    $order_id = pSQL(Tools::getValue('order_id'));

    $result = ShipQuik_BD::getDatabyOrderId($order_id);
    $response = [];

    if (!$result['shipment_data']) {
        $shipmentInfo = json_decode($result['total_payment'], true);

        if ($shipmentInfo) {
            $shipQuikShipments = new ShipQuik_Shipments($module, $order_id, $shipmentInfo['shipment']['boxes']);
            if (!empty($shipQuikShipments->data)) {
                $result = $shipQuikShipments->send();
                $order = new Order((int) $order_id);

                if ($result && $result['status'] == 200) {
                    $sql = ShipQuik_BD::setShipmentDatabyOrder($order, $result);
                    $response = [
                        'status' => 200,
                        'reference' => $result['data']['reference'],
                        'message' => 'Pedido procesado correctamente.',
                    ];
                    $configOrderStatus = $module->config->getValueByKey('orderStatus');
                    if ((int) $configOrderStatus > 0) {
                        $order->setCurrentState((int) $configOrderStatus);
                    }
                    $order->setWsShippingNumber($shipQuikShipments->data['reference']);
                } else {
                    $sql = ShipQuik_BD::setShipmentDatabyOrder($order, ['status' => 500, 'message' => 'ERROR']);
                    $response = [
                        'status' => 200,
                        'message' => 'Ha habido un problema para solicitar la etiqueta, nos pondremos en contacto contigo.',
                    ];
                    $configOrderStatus = $module->config->getValueByKey('orderStatus');
                    if ((int) $configOrderStatus > 0) {
                        $order->setCurrentState((int) $configOrderStatus);
                    }
                }
            }
        }
    } else {
        $response = ['status' => 200, 'message' => 'PROCESSED', 'reference' => $order_id];
    }

    echo json_encode($response);
}
