<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/OSL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author Ship-Quik SL <info@ship-quik.com>
 * @copyright Since 2022 Ship-Quik SL
 * @license   https://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 */
class Ship_QuikActionAdminControllerSetMediaController
{
    protected $module;
    protected $file;
    protected $_path;

    protected $context;

    public function __construct($module, $file, $path)
    {
        $this->file = $file;
        $this->module = $module;
        $this->context = Context::getContext();
        $this->_path = $path;
    }

    public function run($params)
    {
        // Only on AdminOrder page
        if ('AdminOrders' === $this->context->controller->php_self) {
            $this->context->controller->addJS($this->_path . 'views/js/repackaging.min.js', false);
            $this->context->controller->addJS($this->_path . 'views/js/order.min.js', false);

            $this->context->controller->addCSS($this->_path . 'views/css/admin.css', 'all', false);
            $this->context->controller->addCSS('//fonts.googleapis.com/css2?family=Material+Symbols+Outlined:opsz,wght,FILL,GRAD@48,400,0,0', 'all', false);

            $this->context->controller->addJS($this->_path . 'views/js/jquery-ui.min.js', false);
            $this->context->controller->addCSS($this->_path . 'views/css/jquery-ui.css', 'all', false);

            $shop = new Shop((int) $this->context->shop->id);

            $base_url = $shop->getBaseURL(true);

            $ajax = $base_url . 'modules/' . $this->module->name . '/ajax/ajax_send_order.php?token=' . pSQL(Tools::encrypt($this->module->name . '/ajax_send_order.php'));
            Media::addJsDef(['shipquik_order_ajax' => $ajax]);

            $ajax = $base_url . 'modules/' . $this->module->name . '/ajax/ajax_order_pickup.php?token=' . pSQL(Tools::encrypt($this->module->name . '/ajax_order_pickup.php'));
            Media::addJsDef(['shipquik_order_pickup_ajax' => $ajax]);

            $ajax = $base_url . 'modules/' . $this->module->name . '/ajax/ajax_orders_pickup.php?token=' . pSQL(Tools::encrypt($this->module->name . '/ajax_orders_pickup.php'));
            Media::addJsDef(['shipquik_orders_pickup_ajax' => $ajax]);

            $ajax = $base_url . 'modules/' . $this->module->name . '/ajax/ajax_print_label.php?token=' . pSQL(Tools::encrypt($this->module->name . '/ajax_print_label.php'));
            Media::addJsDef(['shipquik_label_ajax' => $ajax]);

            $ajax = $base_url . 'modules/' . $this->module->name . '/ajax/ajax_check_orders.php?token=' . pSQL(Tools::encrypt($this->module->name . '/ajax_check_orders.php'));
            Media::addJsDef(['shipquik_check_orders_ajax' => $ajax]);

            $ajax = $base_url . 'modules/' . $this->module->name . '/ajax/ajax_repackaging.php?token=' . pSQL(Tools::encrypt($this->module->name . '/ajax_repackaging.php'));
            Media::addJsDef(['shipquik_repackaging_ajax' => $ajax]);

            $ajax = $base_url . 'modules/' . $this->module->name . '/ajax/ajax_repackaging_totals.php?token=' . pSQL(Tools::encrypt($this->module->name . '/ajax_repackaging_totals.php'));
            Media::addJsDef(['shipquik_repackaging_totals_ajax' => $ajax]);

            $ajax = $base_url . 'modules/' . $this->module->name . '/ajax/ajax_repackaging_save.php?token=' . pSQL(Tools::encrypt($this->module->name . '/ajax_repackaging_save.php'));
            Media::addJsDef(['shipquik_repackaging_save_ajax' => $ajax]);

            Media::addJsDef(['text_accept' => $this->module->getTranslator()->trans('Accept', [], 'Modules.Shipquik.Shipquik')]);
            Media::addJsDef(['text_cancel' => $this->module->getTranslator()->trans('Cancel', [], 'Modules.Shipquik.Shipquik')]);
            Media::addJsDef(['text_close' => $this->module->getTranslator()->trans('Close', [], 'Modules.Shipquik.Shipquik')]);
            Media::addJsDef(['text_save' => $this->module->getTranslator()->trans('Save', [], 'Modules.Shipquik.Shipquik')]);

            Media::addJsDef(['text_alert' => $this->module->getTranslator()->trans('The selected date is not available, the day will be assigned ', [], 'Modules.Shipquik.Shipquik')]);

            Media::addJsDef(['text_date_correct' => $this->module->getTranslator()->trans('Enter a correct date.', [], 'Modules.Shipquik.Shipquik')]);
            Media::addJsDef(['text_order' => $this->module->getTranslator()->trans('order', [], 'Modules.Shipquik.Shipquik')]);

            Media::addJsDef(['repackaging_translations' => [
                'text_delete_product' => $this->module->getTranslator()->trans('Are you sure you want to delete this product?', [], 'Modules.Shipquik.Shipquik'),
                'text_canceling' => $this->module->getTranslator()->trans('Are you sure you want to cancel?', [], 'Modules.Shipquik.Shipquik'),
                'text_original' => $this->module->getTranslator()->trans('Original', [], 'Modules.Shipquik.Shipquik'),
                'text_repackaging' => $this->module->getTranslator()->trans('Repackaging', [], 'Modules.Shipquik.Shipquik'),
                'text_change_customer' => $this->module->getTranslator()->trans('Charged customer', [], 'Modules.Shipquik.Shipquik'),
                'text_current_cost' => $this->module->getTranslator()->trans('Current cost', [], 'Modules.Shipquik.Shipquik'),
                'text_new_cost' => $this->module->getTranslator()->trans('New cost', [], 'Modules.Shipquik.Shipquik'),
                'text_recalculate' => $this->module->getTranslator()->trans('Recalculate', [], 'Modules.Shipquik.Shipquik'),
                'text_drop_products_here' => $this->module->getTranslator()->trans('Drag products here', [], 'Modules.Shipquik.Shipquik'),
                'text_delete_package' => $this->module->getTranslator()->trans('Are you sure you want to remove this package?', [], 'Modules.Shipquik.Shipquik'),
                'text_specify_qty' => $this->module->getTranslator()->trans('Specify how much you want to pack', [], 'Modules.Shipquik.Shipquik'),

                'text_ajax_problem' => $this->module->getTranslator()->trans('There is a problem processing the data.', [], 'Modules.Shipquik.Shipquik'),
            ]]);

            if (Tools::getValue('ShipQuikPaymentSelectedOrders')) {
                Media::addJsDef(['shipquik_selected_orders' => Tools::htmlentitiesUTF8(Tools::getValue('ShipQuikPaymentSelectedOrders'))]);
            }
            if (Tools::getValue('paymentKey')) {
                $this->context->controller->addJS(STRIPE_URL, false);
                $this->context->controller->addCSS($this->_path . 'views/css/stripe.css', 'all', false);
                $this->context->controller->addJS($this->_path . 'views/js/stripe.min.js', false);

                Media::addJsDef(['stkey' => $this->module->config->getValueByKey('stkey')]);

                Media::addJsDef(['shipquik_selected_orders' => Tools::htmlentitiesUTF8(Tools::getValue('ordersId'))]);
                Media::addJsDef(['paymentTotal' => Tools::htmlentitiesUTF8(Tools::getValue('paymentTotal'))]);
                Media::addJsDef(['paymentEmail' => Tools::htmlentitiesUTF8(Tools::getValue('paymentEmail'))]);
                Media::addJsDef(['paymentKey' => Tools::htmlentitiesUTF8(Tools::getValue('paymentKey'))]);
                Media::addJsDef(['paymentReturnSubmitUrl' => Tools::htmlentitiesUTF8(Tools::getValue('paymentReturnSubmitUrl'))]);
                Media::addJsDef(['paymentReturnCancelUrl' => Tools::htmlentitiesUTF8(Tools::getValue('paymentReturnCancelUrl'))]);
            }
            if (Tools::getValue('processShipQuikOrdersModal')) {
                $this->context->controller->addJS($this->_path . 'views/js/process-orders.min.js', false);

                Media::addJsDef(['text_order_previous_processed' => $this->module->getTranslator()->trans('It has already been processed before.', [], 'Modules.Shipquik.Shipquik')]);
                Media::addJsDef(['text_order_pending' => $this->module->getTranslator()->trans('Order pending processing', [], 'Modules.Shipquik.Shipquik')]);

                Media::addJsDef(['returnUrl' => Tools::htmlentitiesUTF8(base64_decode(Tools::getValue('returnUrl')))]);
                Media::addJsDef(['shipquik_selected_orders' => Tools::htmlentitiesUTF8(Tools::getValue('processed_orders_ids'))]);
            }
            if (Tools::getValue('processShipQuikCheckOrders')) {
                $this->context->controller->addJS($this->_path . 'views/js/check-orders.min.js', false);

                Media::addJsDef(['returnUrl' => Tools::htmlentitiesUTF8(base64_decode(Tools::getValue('returnUrl')))]);
                Media::addJsDef(['shipquik_selected_orders' => Tools::htmlentitiesUTF8(Tools::getValue('processed_orders_ids'))]);
            }
            if (Tools::getValue('labelsShipQuikOrdersModal')) {
                $this->context->controller->addJS($this->_path . 'views/js/pdf-lib.min.js', false);
                $this->context->controller->addCSS($this->_path . 'views/css/print.min.css', 'all', false);
                $this->context->controller->addJS($this->_path . 'views/js/print.min.js', false);
                $this->context->controller->addJS($this->_path . 'views/js/labels.min.js', false);

                Media::addJsDef(['returnUrl' => Tools::htmlentitiesUTF8(Tools::getValue('returnUrl'))]);
                Media::addJsDef(['printProducts' => Tools::htmlentitiesUTF8(Tools::getValue('printProducts'))]);
                Media::addJsDef(['shipquik_selected_orders' => Tools::htmlentitiesUTF8(Tools::getValue('ordersId'))]);
            }
            if (Tools::getValue('pickupShipQuikOrdersModal')) {
                $this->context->controller->addJS($this->_path . 'views/js/pickup.min.js', false);

                Media::addJsDef(['text_morning' => $this->module->getTranslator()->trans('Morning', [], 'Modules.Shipquik.Shipquik')]);
                Media::addJsDef(['text_afternoon' => $this->module->getTranslator()->trans('Afternoon', [], 'Modules.Shipquik.Shipquik')]);

                Media::addJsDef(['min_date' => date('Y-m-d')]);
                Media::addJsDef(['returnUrl' => Tools::htmlentitiesUTF8(base64_decode(Tools::getValue('returnUrl')))]);
                Media::addJsDef(['shipquik_selected_orders' => Tools::htmlentitiesUTF8(Tools::getValue('processed_orders_ids'))]);
            }
        } elseif (Tools::getValue('configure') == $this->module->name) {
            $this->context->controller->addJS($this->_path . 'views/js/settings-form.min.js', false);
            $this->context->controller->addJS($this->_path . 'views/js/settings-tablerates.min.js', false);
            $this->context->controller->addJS($this->_path . 'views/js/settings-packagelist.min.js', false);
            $this->context->controller->addCSS($this->_path . 'views/css/settings.css', 'all', false);
        }
    }
}
