<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/OSL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author Ship-Quik SL <info@ship-quik.com>
 * @copyright Since 2022 Ship-Quik SL
 * @license   https://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 */

use PrestaShop\PrestaShop\Core\Grid\Data\GridData;
use PrestaShop\PrestaShop\Core\Grid\Record\RecordCollection;
use ShipQuik\PrestaShop\ShipQuik_BD;

class Ship_QuikActionOrderGridDataModifierController
{
    protected $module;
    protected $file;
    protected $_path;

    protected $context;

    public function __construct($module, $file, $path)
    {
        $this->file = $file;
        $this->module = $module;
        $this->context = Context::getContext();
        $this->_path = $path;
    }

    public function run($params)
    {
        if (empty($params['data'])) {
            return;
        }

        if (Tools::getValue('redirect_status') && Tools::getValue('redirect_status') == 'succeeded') {
            $processedOrderID = explode(',', Tools::getValue('processed_orders_ids'));
            foreach ($processedOrderID as $order_id) {
                $order = new Order($order_id);
                ShipQuik_BD::setPaidStatus($order, '1');
            }
        }

        $gridData = $params['data'];
        $modifiedRecords = $gridData->getRecords()->all();

        foreach ($modifiedRecords as $key => $data) {
            $order_id = $data['id_order'];
            $order = new Order((int) $order_id);

            $result = ShipQuik_BD::getDatabyOrder($order);

            $modifiedRecords[$key]['shipquik_has_shipment_data'] = false;

            if ($result) {
                $shipmentInfo = json_decode(str_replace('&quot;', '"', $result['shipment_data']), true, 99);
                $totalPayment = json_decode(str_replace('&quot;', '"', $result['total_payment']), true, 99);

                if (isset($totalPayment['payment']['total'])) {
                    $modifiedRecords[$key]['shipquik_total_payment'] = (float) $totalPayment['payment']['total'];
                }
                if (isset($shipmentInfo['data'])) {
                    $modifiedRecords[$key]['shipquik_has_shipment_data'] = true;
                    $modifiedRecords[$key]['shipquik_reference'] = $shipmentInfo['data']['reference'];
                }

                $modifiedRecords[$key]['shipquik_paid'] = $result['paid'];
                $modifiedRecords[$key]['shipquik_printed'] = $result['printed'];
                $modifiedRecords[$key]['currency_sign'] = $this->context->currency->sign;
                $modifiedRecords[$key]['min_date'] = date('Y-m-d');

                $modifiedRecords[$key]['text_morning'] = $this->module->getTranslator()->trans('Morning', [], 'Modules.Shipquik.Shipquik');
                $modifiedRecords[$key]['text_afternoon'] = $this->module->getTranslator()->trans('Afternoon', [], 'Modules.Shipquik.Shipquik');

                $modifiedRecords[$key]['text_paid'] = $this->module->getTranslator()->trans('Paid', [], 'Modules.Shipquik.Shipquik');
                $modifiedRecords[$key]['text_unpaid'] = $this->module->getTranslator()->trans('Unpaid', [], 'Modules.Shipquik.Shipquik');

                if (isset($result['pickup_datetime'])) {
                    $modifiedRecords[$key]['shipquik_pickup_datetime'] = json_decode($result['pickup_datetime'], true);
                }

                if (isset($shipmentInfo['message'])) {
                    $modifiedRecords[$key]['shipquik_message'] = $shipmentInfo['message'];
                } else {
                    $modifiedRecords[$key]['shipquik_message'] = '';
                }

                unset($shipmentInfo);
                unset($totalPayment);
                unset($order);
                unset($result);
            }
        }

        $params['data'] = new GridData(
            new RecordCollection($modifiedRecords),
            $gridData->getRecordsTotal(),
            $gridData->getQuery()
        );
    }
}
