<?php
/**
 * 2022 Ship-Quik
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Apache License 2.0
 * that is bundled with this package in the file LICENSE.
 * It is also available through the world-wide-web at this URL:
 * http://www.apache.org/licenses/LICENSE-2.0.txt
 *
 * @author    Ship-Quik <info@ship-quik.com>
 * @copyright 2022 Ship-Quik S.L.
 * @license   http://www.apache.org/licenses/LICENSE-2.0.txt  Apache License 2.0
 */
if (!defined('_PS_VERSION_')) {
    exit;
}
define('SHIP_QUIK_PLATFORM', 'prestashop');

define('SHIP_QUIK_NAME', 'ship_quik');
define('SHIP_QUIK_VERSION', '1.0.0');
define('SHIP_QUIK_PLUGIN_PATH', rtrim(_PS_MODULE_DIR_, '/') . '/' . SHIP_QUIK_NAME . '/');
define('SHIP_QUIK_CLASSES_PATH', SHIP_QUIK_PLUGIN_PATH . 'ShipQuik/');

define('CONFIG_DATA', SHIP_QUIK_PLUGIN_PATH . 'config.json');

define('STRIPE_URL', 'https://js.stripe.com/v3/');
define('SHIP_QUIK_URL', 'https://ship-quik.com/es');
define('SHIP_QUIK_URL_PLUGINS', 'https://ship-quik.com/es/plugins');

// Load composer
require SHIP_QUIK_PLUGIN_PATH . 'vendor/autoload.php';

use ShipQuik\Shared\ShipQuik_Configuration;
use ShipQuik\Shared\ShipQuik_Helper;

class Ship_Quik extends CarrierModule
{
    /**
     * Id of current carrier. This variable will be set by Presta when
     * method for calculating shipping cost is called.
     *
     * @var int
     */
    public $id_carrier;

    public $_token = '';
    public $config;
    public $_suppliers;

    public $_is_required;

    /**
     * ShipQuik constructor.
     */
    public function __construct()
    {
        $this->module_key = 'ca6b740c3dd6e6070fae9fb81fc4811d';
        $this->name = 'ship_quik';
        $this->tab = 'shipping_logistics';
        $this->version = '1.0.0';
        $this->author = 'Ship-Quik S.L.';
        $this->need_instance = 0;
        $this->bootstrap = true;

        parent::__construct();

        $this->_token = strtoupper('ship_quik');
        $this->config = new ShipQuik_Configuration($this);

        $this->displayName = $this->trans('Ship-Quik Shipping', [], 'Modules.Shipquik.Shipquik');
        $this->description = $this->trans('Make easy your job, comparing transport rates and adjusting shipments to your needs. Save time and money in a super easy way.', [], 'Modules.Shipquik.Shipquik');

        $this->confirmUninstall = $this->trans('Are you sure you want to uninstall?', [], 'Modules.Shipquik.Shipquik');

        if (!Configuration::get('SHIP_QUIK_NAME')) {
            $this->warning = $this->trans('No name provided', [], 'Modules.Shipquik.Shipquik');
        }

        $this->_is_required = $this->checkRequiredFields();
    }

    public function install()
    {
        if (Shop::isFeatureActive()) {
            Shop::setContext(Shop::CONTEXT_ALL);
        }

        $sql_file = dirname(__FILE__) . '/install/install.sql';
        if (!$this->loadSQLFile($sql_file)) {
            return false;
        }

        if (!parent::install()) {
            return false;
        }

        // && $this->registerHook('displayLeftColumn')
        // && $this->registerHook('displayHeader')
        // || !$this->registerHook('actionOrderGridPresenterModifier')

        Configuration::updateValue('SHIP_QUIK_NAME', 'Ship Quik');

        if (!$this->registerHook('actionCarrierUpdate')
            || !$this->registerHook('displayCarrierExtraContent')
            || !$this->registerHook('displayOrderConfirmation')
            || !$this->registerHook('actionAdminControllerSetMedia')
            || !$this->registerHook('actionOrderGridDefinitionModifier')
            || !$this->registerHook('actionOrderGridDataModifier')
            || !$this->registerHook('actionOrderGridQueryBuilderModifier')
            || !$this->registerHook('actionAdminOrdersListingFieldsModifier')
            || !$this->registerHook('actionAdminOrdersListingResultsModifier')) {
            return false;
        }

        Tools::clearAllCache();

        return true;
    }

    public function enable($force_all = false)
    {
        $this->installCarriers(Configuration::get('_SHIP_QUIK_pricingMode'));

        return parent::enable($force_all);
    }

    public function disable($force_all = false)
    {
        $carriers_list = [
            '_SHIP_QUIK_SPEED' => [
                'name' => $this->trans('Ship-Quik faster shipping', [], 'Modules.Shipquik.Shipquik'),
                'active' => 0,
            ],
            '_SHIP_QUIK_PRICE' => [
                'name' => $this->trans('Ship-Quik cheaper shipping', [], 'Modules.Shipquik.Shipquik'),
                'active' => 0,
            ],
        ];

        foreach ($carriers_list as $carrier_key => $carrierData) {
            $existCarrierID = Configuration::get($carrier_key);
            if ($existCarrierID) {
                $existCarrier = new Carrier($existCarrierID);
                $existCarrier->active = $carrierData['active'];
                $existCarrier->save();
            }
        }

        return parent::disable($force_all);
    }

    public function uninstall()
    {
        return parent::uninstall()
            && Configuration::deleteByName('SHIP_QUIK_NAME')
            && $this->uninstallModule();
    }

    public function uninstallModule()
    {
        $sql_file = dirname(__FILE__) . '/install/uninstall.sql';
        if (!$this->loadSQLFile($sql_file)) {
            return false;
        }

        return true;
    }

    public function loadSQLFile($sql_file)
    {
        $sql_content = Tools::file_get_contents($sql_file);

        $sql_content = str_replace('PREFIX_', _DB_PREFIX_, $sql_content);
        $sql_requests = preg_split("/;\s*[\r\n]+/", $sql_content);

        $result = true;
        foreach ($sql_requests as $request) {
            if (!empty($request)) {
                $result &= Db::getInstance()->execute(trim($request));
            }
        }

        return $result;
    }

    public function getHookController($hook_name)
    {
        require_once dirname(__FILE__) . '/controllers/hook/' . $hook_name . '.php';
        $controller_name = $this->name . $hook_name . 'Controller';
        $controller = new $controller_name($this, __FILE__, $this->_path);

        return $controller;
    }

    public function isUsingNewTranslationSystem()
    {
        return true;
    }

    public function getSupplierServices($name)
    {
        foreach ($this->_suppliers as $supplier) {
            if ($supplier['name'] == $name) {
                return $supplier['services'];
            }
        }

        return [];
    }

    public function replaceStrInArray($search, $replace, $list)
    {
        $result = [];

        foreach ($list as $row) {
            $tmp = [];
            foreach ($row as $key => $element) {
                $tmp[$key] = str_replace($search, $replace, $element);
            }
            $result[] = $tmp;
        }

        return $result;
    }

    public function getContent()
    {
        $output = '';
        $submitName = $this->name;

        if (!Configuration::get('_SHIP_QUIK_activation')) {
            $correct_global = true;

            if (Tools::isSubmit('submit' . $submitName . '_activation')) {
                foreach ($this->getActivationFormFields() as $field => $values) {
                    $correct = true;

                    if ($field == '_SHIP_QUIK_activationEmail') {
                        $configValue = Tools::getValue($field);
                        if ($values['required']) {
                            if (!$configValue) {
                                $correct = false;
                                $correct_global = false;
                                $output = $this->displayError($this->trans('You must enter a value for ' . $field, [], 'Modules.Shipquik.Shipquik'));
                            }
                        }
                        if ($correct) {
                            Configuration::updateValue('_SHIP_QUIK_activationEmail', $configValue);
                        } else {
                            break;
                        }
                    }
                    if ($field == '_SHIP_QUIK_activationKey') {
                        $configValue = Tools::getValue($field);
                        if ($values['required']) {
                            if (!$configValue) {
                                $correct = false;
                                $correct_global = false;
                                $output = $this->displayError($this->trans('You must enter a value for ' . $field, [], 'Modules.Shipquik.Shipquik'));
                            }
                        }
                        if ($correct) {
                            Configuration::updateValue('_SHIP_QUIK_activationKey', $configValue);
                            $correct_global = true;
                        } else {
                            $correct_global = false;
                            $output = $this->displayError($this->trans('You must enter a value for ' . $field, [], 'Modules.Shipquik.Shipquik'));
                        }
                    }
                }

                if ($correct_global) {
                    $output = $this->displayConfirmation($this->trans('Configuration saved successfully', [], 'Modules.Shipquik.Shipquik'));

                    $this->config->reNewAccessToken();
                    $token = unserialize(Configuration::get('_SHIP_QUIK_activation'));

                    if (isset($token->token) && $token->token) {
                        $redirect = Context::getContext()->shop->getBaseURL(true) . end(explode('/', _PS_ADMIN_DIR_)) . '/'
                        . AdminController::$currentIndex
                        . '&' . http_build_query([
                            'configure' => $submitName,
                            'token' => Tools::getValue('token'),
                        ]);

                        Tools::redirect($redirect);
                    }
                    $output = $this->displayError($this->trans('There was a problem with the activation, please check that the private key is correct.', [], 'Modules.Shipquik.Shipquik'));
                }
            }

            return $output . $this->displayActivationForm($submitName);
        } else {
            $correct_global = true;

            $this->_suppliers = ShipQuik_Helper::getSuppliersConfiguration($this, []);

            if (Tools::isSubmit('submit' . $submitName)) {
                foreach ($this->getFormFields() as $field => $values) {
                    $correct = true;

                    if ($field == '_SHIP_QUIK_suppliers') {
                        $all_opts = $this->getSuppliersOptions();
                        $checkbox_options = [];
                        foreach ($all_opts as $chbx_options) {
                            $value = Tools::getValue('_SHIP_QUIK_suppliers_' . $chbx_options['id_option']);
                            if ($value) {
                                $checkbox_options[] = [
                                    'name' => $chbx_options['id_option'],
                                    'services' => $this->getSupplierServices($chbx_options['id_option']),
                                ];
                            }
                        }
                        $configValue = serialize($checkbox_options);
                    } elseif ($field == '_SHIP_QUIK_shippingTableRates' || $field == '_SHIP_QUIK_shippingPackageList') {
                        $configValue = Tools::getValue($field);
                        if ($values['required']) {
                            if (!$configValue) {
                                $correct = false;
                                $correct_global = false;
                                $output = $this->displayError($this->trans('You must enter a value for ' . $field, [], 'Modules.Shipquik.Shipquik'));
                            }
                        }
                        if ($configValue) {
                            $configValue = $this->replaceStrInArray(',', '.', $configValue);
                        }
                        Configuration::updateValue($field, serialize($configValue));
                        continue;
                    } else {
                        $configValue = (string) Tools::getValue($field);
                    }

                    if ($values['required']) {
                        switch ($values['type']) {
                            case 'serialized':
                                if (empty(unserialize($configValue))) {
                                    $correct = false;
                                    $correct_global = false;
                                    $output = $this->displayError($this->trans('You must enter a value for ' . $field, [], 'Modules.Shipquik.Shipquik'));
                                }
                                break;

                            case 'select':
                                if (empty($configValue) && $configValue != '0') {
                                    $correct = false;
                                    $correct_global = false;
                                    $output = $this->displayError($this->trans('You must enter a value for ' . $field, [], 'Modules.Shipquik.Shipquik'));
                                }
                                break;
                            case 'number':
                                if ($configValue <= 0 || !is_numeric($configValue)) {
                                    $correct = false;
                                    $correct_global = false;
                                    $output = $this->displayError($this->trans('You must enter a value for ' . $field, [], 'Modules.Shipquik.Shipquik'));
                                } else {
                                    $configValue = str_replace(',', '.', $configValue);
                                }
                                break;
                            case 'text':
                                if (empty($configValue)) {
                                    $correct = false;
                                    $correct_global = false;
                                    $output = $this->displayError($this->trans('You must enter a value for ' . $field, [], 'Modules.Shipquik.Shipquik'));
                                }
                                break;
                        }
                    }
                    if ($correct) {
                        Configuration::updateValue($field, $configValue);
                    } else {
                        break;
                    }
                }

                if ($correct_global) {
                    $this->installCarriers(Configuration::get('_SHIP_QUIK_pricingMode'));

                    $output = $this->displayConfirmation($this->trans('Configuration saved successfully', [], 'Modules.Shipquik.Shipquik'));
                }
            }
        }

        return $output . $this->displayFullForm($submitName);
    }

    public function displayActivationForm($submitName)
    {
        $activation_message = $this->get('twig')->render('@Modules/ship_quik/views/templates/admin/activation_message.twig',
            [
                'shipquik_url' => SHIP_QUIK_URL,
                'shipquik_url_plugins' => SHIP_QUIK_URL_PLUGINS,
            ]);

        $form1 = [
            'form' => [
                'legend' => [
                    'title' => $this->trans('IMPORTANT', [], 'Modules.Shipquik.Shipquik'),
                ],
                'input' => [
                    [
                        'type' => 'html',
                        'name' => 'info',
                        'label' => '',
                        'class' => 'panel-activation',
                        'html_content' => $activation_message,
                        ],
                ],
            ],
        ];

        $form2 = [
            'form' => [
                'legend' => [
                    'title' => $this->trans('Ship-Quik Licensing and Activation', [], 'Modules.Shipquik.Shipquik'),
                ],
                'input' => [
                    [
                        'type' => 'text',
                        'label' => $this->trans('Registration email', [], 'Modules.Shipquik.Shipquik'),
                        'desc' => $this->trans('Enter here the email with which you registered with Ship-Quik.', [], 'Modules.Shipquik.Shipquik'),
                        'name' => '_SHIP_QUIK_activationEmail',
                        'required' => true,
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->trans('Private key', [], 'Modules.Shipquik.Shipquik'),
                        'desc' => $this->trans('Enter your private key here.', [], 'Modules.Shipquik.Shipquik'),
                        'name' => '_SHIP_QUIK_activationKey',
                        'required' => true,
                    ],
                ],
                'submit' => [
                    'title' => $this->trans('Activate', [], 'Modules.Shipquik.Shipquik'),
                    'class' => 'btn btn-default pull-right',
                ],
            ],
        ];

        $helper = new HelperForm();

        $helper->table = $this->name;
        $helper->name_controller = $submitName;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex . '&' . http_build_query(['configure' => $submitName]);
        $helper->submit_action = 'submit' . $submitName . '_activation';

        $helper->default_form_language = (int) Configuration::get('PS_LANG_DEFAULT');

        $helper->fields_value['_SHIP_QUIK_activationEmail'] = '';
        $helper->fields_value['_SHIP_QUIK_activationKey'] = '';

        return $helper->generateForm([$form1, $form2]);
    }

    public function installCarriers($pricingMode)
    {
        $carriers_list = [
            '_SHIP_QUIK_SPEED' => [
                'name' => $this->trans('Ship-Quik faster shipping', [], 'Modules.Shipquik.Shipquik'),
                'active' => ($pricingMode == 'speed' || $pricingMode == 'speedprice' ? 1 : 0),
            ],
            '_SHIP_QUIK_PRICE' => [
                'name' => $this->trans('Ship-Quik cheaper shipping', [], 'Modules.Shipquik.Shipquik'),
                'active' => ($pricingMode == 'price' || $pricingMode == 'speedprice' ? 1 : 0),
            ],
        ];

        $id_lang_default = Language::getIsoById(Configuration::get('PS_LANG_DEFAULT'));
        foreach ($carriers_list as $carrier_key => $carrierData) {
            $existCarrierID = Configuration::get($carrier_key);
            if ($existCarrierID) {
                $existCarrier = new Carrier($existCarrierID);
                $existCarrier->name = $carrierData['name'];
                $existCarrier->active = $carrierData['active'];
                foreach (Language::getLanguages(true) as $language) {
                    $existCarrier->delay[(int) $language['id_lang']] = Configuration::get('_SHIP_QUIK_pickUpTimeText');
                }
                $existCarrier->save();
            } else {
                // Create new carrier
                $carrier = new Carrier();
                $carrier->name = $carrierData['name'];
                $carrier->id_tax_rules_group = 0;
                $carrier->active = $carrierData['active'];
                $carrier->deleted = 0;
                foreach (Language::getLanguages(true) as $language) {
                    $carrier->delay[(int) $language['id_lang']] = Configuration::get('_SHIP_QUIK_pickUpTimeText');
                }
                $carrier->shipping_handling = false;
                $carrier->range_behavior = 1;
                $carrier->is_module = true;
                $carrier->shipping_external = true;
                $carrier->external_module_name = $this->name;
                $carrier->shipping_method = 1;
                $carrier->need_range = true;
                if (!$carrier->add()) {
                    return false;
                }

                // Associate carrier to all groups
                $groups = Group::getGroups(true);
                foreach ($groups as $group) {
                    Db::getInstance()->insert('carrier_group', ['id_carrier' => (int) $carrier->id, 'id_group' => (int) $group['id_group']]);
                }

                // Create price range
                $rangePrice = new RangePrice();
                $rangePrice->id_carrier = $carrier->id;
                $rangePrice->delimiter1 = '0';
                $rangePrice->delimiter2 = '999999';
                $rangePrice->add();

                // Create weight range
                $rangeWeight = new RangeWeight();
                $rangeWeight->id_carrier = $carrier->id;
                $rangeWeight->delimiter1 = '0';
                $rangeWeight->delimiter2 = '999999';
                $rangeWeight->add();

                // Associate carrier to all zones
                $zones = Zone::getZones(true);
                foreach ($zones as $zone) {
                    Db::getInstance()->insert('carrier_zone', ['id_carrier' => (int) $carrier->id, 'id_zone' => (int) $zone['id_zone']]);
                    Db::getInstance()->insert('delivery', ['id_carrier' => (int) $carrier->id, 'id_range_price' => (int) $rangePrice->id, 'id_range_weight' => null, 'id_zone' => (int) $zone['id_zone'], 'price' => '0']);
                    Db::getInstance()->insert('delivery', ['id_carrier' => (int) $carrier->id, 'id_range_price' => null, 'id_range_weight' => (int) $rangeWeight->id, 'id_zone' => (int) $zone['id_zone'], 'price' => '0']);
                }

                // Save the carrier ID in the Configuration table
                Configuration::updateValue($carrier_key, $carrier->id);
            }
        }

        return true;
    }

    public function getSuppliersOptions()
    {
        $options = [];

        if ($this->_suppliers) {
            foreach ($this->_suppliers as $supplier) {
                $options[] = [
                    'id_option' => $supplier['name'],
                    'name' => $this->trans($supplier['name'], [], 'Modules.Shipquik.Shipquik'),
                ];
            }
        }

        return $options;
    }

    public function getPricingMode()
    {
        $options = [
            [
                'id_option' => 'price',
                'name' => $this->trans('Faster shipping', [], 'Modules.Shipquik.Shipquik'),
            ],
            [
                'id_option' => 'speed',
                'name' => $this->trans('Cheaper shipping', [], 'Modules.Shipquik.Shipquik'),
            ],
            [
                'id_option' => 'speedprice',
                'name' => $this->trans('The customer can choose between the cheapest or the fastest', [], 'Modules.Shipquik.Shipquik'),
            ],
        ];

        return $options;
    }

    public function getPaymentMethod()
    {
        $options = [];

        if ($this->config->getPaymentMethod() == 0) {
            $options[] = [
                'id_option' => 0,
                'name' => $this->trans('End of the month', [], 'Modules.Shipquik.Shipquik'),
            ];
        } else {
            $options[] = [
                'id_option' => 1,
                'name' => $this->trans('Credit card', [], 'Modules.Shipquik.Shipquik'),
            ];
        }

        return $options;
    }

    public function getFreeShipping()
    {
        $options = [
            [
                'id_option' => 'no',
                'name' => $this->trans('No', [], 'Modules.Shipquik.Shipquik'),
            ],
            [
                'id_option' => 'yes',
                'name' => $this->trans('Yes', [], 'Modules.Shipquik.Shipquik'),
            ],
        ];

        return $options;
    }

    public function getPrintProducts()
    {
        $options = [
            [
                'id_option' => 'no',
                'name' => $this->trans('No', [], 'Modules.Shipquik.Shipquik'),
            ],
            [
                'id_option' => 'yes',
                'name' => $this->trans('Yes', [], 'Modules.Shipquik.Shipquik'),
            ],
        ];

        return $options;
    }

    public function getOrderStatus()
    {
        $result = [];
        $result[] = [
            'id_option' => '0',
            'name' => $this->trans('Do nothing', [], 'Modules.Shipquik.Shipquik'),
        ];
        foreach (OrderStateCore::getOrderStates(Context::getContext()->language->id) as $order_status) {
            $result[] = [
                'id_option' => $order_status['id_order_state'],
                'name' => $order_status['name'],
            ];
        }

        return $result;
    }

    public function getChargeDUA()
    {
        $options = [
            [
                'id_option' => 'yes',
                'name' => $this->trans('Yes', [], 'Modules.Shipquik.Shipquik'),
            ],
            [
                'id_option' => 'no',
                'name' => $this->trans('No', [], 'Modules.Shipquik.Shipquik'),
            ],
        ];

        return $options;
    }

    public function getShopFeeType()
    {
        $options = [
            [
                'id_option' => 'price',
                'name' => $this->trans('By price', [], 'Modules.Shipquik.Shipquik'),
            ],
            [
                'id_option' => 'percent',
                'name' => $this->trans('By percent', [], 'Modules.Shipquik.Shipquik'),
            ],
        ];

        return $options;
    }

    public function getEditableTable($id, $description, $data)
    {
        $data = Configuration::get('_SHIP_QUIK_shippingTableRates');

        if (!$data) {
            $data = [];
        } else {
            $data = unserialize($data);
        }

        $html = $this->get('twig')->render('@Modules/ship_quik/views/templates/admin/table_rates.twig',
            [
                'data' => $data ? $data : [],
                'id' => $id,
                'description' => $description,
                'data_count' => count($data ? $data : []),
                'add_button' => $this->trans('Add rate', [], 'Modules.Shipquik.Shipquik'),
                'delete_text' => $this->trans('Are you sure you want to delete this range?', [], 'Modules.Shipquik.Shipquik'),
                ]
        );

        return $html;
    }

    public function getProductListWithoutParcel()
    {
        $html = '';
        $limit = 15;

        $products = Product::getProducts(Context::getContext()->language->getId(), 1, -1, 'id_product', 'ASC');

        $productsNeedParcelData = [];

        foreach ($products as $element) {
            $product = new Product($element['id_product']);

            $name = Product::getProductName($element['id_product']);
            $link = Context::getContext()->link->getAdminLink('AdminProducts', true, ['id_product' => $element['id_product']]);

            $width = (float) $element['width'];
            $height = (float) $element['height'];
            $depth = (float) $element['depth'];
            $weight = (float) $element['weight'];

            if ($weight == 0 || $depth == 0 || $width == 0 || $height == 0) {
                $productsNeedParcelData[] = [
                    'name' => $name,
                    'width' => $width,
                    'height' => $height,
                    'depth' => $depth,
                    'weight' => $weight,
                    'link' => $link,
                ];
            }
        }

        if (count($productsNeedParcelData) > 0) {
            $html = $this->get('twig')->render('@Modules/ship_quik/views/templates/admin/product_list.twig',
                [
                    'data' => $productsNeedParcelData,
                    'alert' => $this->trans('Attention: the packaging data of these products is not fully reported.', [], 'Modules.Shipquik.Shipquik'),
                    'description' => $this->trans('We only show the ' . $limit . ' first, check them out and come back to this page for more products', [], 'Modules.Shipquik.Shipquik'),
                    'product_name' => $this->trans('Product name', [], 'Modules.Shipquik.Shipquik'),
                    'product_width' => $this->trans('Width', [], 'Modules.Shipquik.Shipquik'),
                    'product_height' => $this->trans('Height', [], 'Modules.Shipquik.Shipquik'),
                    'product_depth' => $this->trans('Depth', [], 'Modules.Shipquik.Shipquik'),
                    'product_weight' => $this->trans('Weight', [], 'Modules.Shipquik.Shipquik'),
                    'product_link' => $this->trans('Product page', [], 'Modules.Shipquik.Shipquik'),
                    'product_details' => $this->trans('More', [], 'Modules.Shipquik.Shipquik'),
                    ]
            );
        }

        return $html;
    }

    public function getPackageList($id, $description, $data)
    {
        $html = '';
        $packageList = [];
        $data = Configuration::get('_SHIP_QUIK_shippingPackageList');

        if (!$data) {
            $data = [];
        } else {
            $data = unserialize($data);
        }

        if ($data) {
            foreach ($data as $element) {
                $packageList[] = [
                    'name' => $element['name'],
                    'depth' => $element['depth'],
                    'width' => $element['width'],
                    'height' => $element['height'],
                    'weight' => $element['weight'],
                    'default' => (isset($element['default']) && $element['default'] == 'on' ? 'checked=checked' : ''),
                ];
            }
        }

        $html = $this->get('twig')->render('@Modules/ship_quik/views/templates/admin/package_list.twig',
            [
                'data' => $packageList ? $packageList : [],
                'id' => $id,
                'description' => $description,
                'data_count' => count($packageList ? $packageList : []),
                'add_button' => $this->trans('Add package', [], 'Modules.Shipquik.Shipquik'),
                'delete_text' => $this->trans('Are you sure you want to delete this package?', [], 'Modules.Shipquik.Shipquik'),
                'package_name' => $this->trans('Name', [], 'Modules.Shipquik.Shipquik'),
                'package_depth' => $this->trans('Depth (cm)', [], 'Modules.Shipquik.Shipquik'),
                'package_width' => $this->trans('Width (cm)', [], 'Modules.Shipquik.Shipquik'),
                'package_height' => $this->trans('Height (cm)', [], 'Modules.Shipquik.Shipquik'),
                'package_weight' => $this->trans('Weight (kgr)', [], 'Modules.Shipquik.Shipquik'),
                'package_default' => $this->trans('Default package', [], 'Modules.Shipquik.Shipquik'),
            ]
        );

        return $html;
    }

    public function displayFullForm($submitName)
    {
        $logo = Tools::file_get_contents(SHIP_QUIK_PLUGIN_PATH . 'views/img/logos/logo.base64');

        $form1 = [
            'form' => [
                'legend' => [
                    'title' => $this->trans('Carriers List', [], 'Modules.Shipquik.Shipquik'),
                ],
                'input' => [
                    [
                        'type' => 'checkbox',
                        'desc' => $this->trans('Choose which carriers you want to work with', [], 'Modules.Shipquik.Shipquik'),
                        'name' => '_SHIP_QUIK_suppliers',
                        'required' => true,
                        'values' => [
                            'query' => $this->getSuppliersOptions(),
                            'id' => 'id_option',
                            'name' => 'name',
                            'expand' => [
                                'print_total' => count($this->getSuppliersOptions()),
                                'default' => 'show',
                                'show' => ['text' => $this->trans('show', [], 'Modules.Shipquik.Shipquik'), 'icon' => 'plus-sign-alt'],
                                'hide' => ['text' => $this->trans('hide', [], 'Modules.Shipquik.Shipquik'), 'icon' => 'minus-sign-alt'],
                            ],
                        ],
                    ],
                ],
            ],
        ];

        $form2 = [
            'form' => [
                'legend' => [
                    'title' => $this->trans('Payment method', [], 'Modules.Shipquik.Shipquik'),
                ],
                'input' => [
                    [
                        'type' => 'select',
                        'name' => '_SHIP_QUIK_paymentMethod',
                        'desc' => 'This option will allow you to pay for your orders at the end of the month. By clicking, we will receive an email with your request. You will have to sign the SEPA document and when it is authorized, you will be able to pay at the end of the month. If you want more information send us an email to <a href="mailto:vamos@ship-quik.com">vamos@ship-quik.com</a>',
                        'required' => true,
                        'options' => [
                            'query' => $this->getPaymentMethod(),
                            'id' => 'id_option',
                            'name' => 'name',
                        ],
                    ],
                ],
            ],
        ];

        $form3 = [
            'form' => [
                'legend' => [
                    'title' => $this->trans('More options', [], 'Modules.Shipquik.Shipquik'),
                ],
                'input' => [
                    [
                        'type' => 'select',
                        'label' => $this->trans('Price selection', [], 'Modules.Shipquik.Shipquik'),
                        'name' => '_SHIP_QUIK_pricingMode',
                        'required' => true,
                        'class' => 'subsection',
                        'options' => [
                            'query' => $this->getPricingMode(),
                            'id' => 'id_option',
                            'name' => 'name',
                        ],
                    ],
                    [
                        'type' => 'select',
                        'label' => $this->trans('Charge import costs to your customers', [], 'Modules.Shipquik.Shipquik'),
                        'name' => '_SHIP_QUIK_chargeDUA',
                        'desc' => $this->trans('Do you want to charge import costs to your customers? If Yes, this cost will be charged even in free shipments.', [], 'Modules.Shipquik.Shipquik'),
                        'required' => true,
                        'options' => [
                            'query' => $this->getChargeDUA(),
                            'id' => 'id_option',
                            'name' => 'name',
                        ],
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->trans('Text in Transit Time', [], 'Modules.Shipquik.Shipquik'),
                        'name' => '_SHIP_QUIK_pickUpTimeText',
                        'required' => true,
                    ],
                    [
                        'type' => 'select',
                        'label' => $this->trans('Free shipping', [], 'Modules.Shipquik.Shipquik'),
                        'name' => '_SHIP_QUIK_isFreeShipping',
                        'desc' => $this->trans('If you want to offer free shipping to your customers', [], 'Modules.Shipquik.Shipquik'),
                        'required' => false,
                        'options' => [
                            'query' => $this->getFreeShipping(),
                            'id' => 'id_option',
                            'name' => 'name',
                        ],
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->trans('Free shipping from', [], 'Modules.Shipquik.Shipquik'),
                        'name' => '_SHIP_QUIK_freeShippingPriceFrom',
                        'desc' => $this->trans('Cart subtotal equal to or greater than this amount', [], 'Modules.Shipquik.Shipquik'),
                        'required' => false,
                    ],
                    [
                        'type' => 'html',
                        'label' => $this->trans('Rates by range of amounts', [], 'Modules.Shipquik.Shipquik'),
                        'name' => '_SHIP_QUIK_shippingTableRates',
                        'desc' => 'Define your rate depending on the cart subtotal',
                        'html_content' => $this->getEditableTable('_SHIP_QUIK_shippingTableRates', '', Configuration::get('_SHIP_QUIK_shippingTableRates')),
                        'required' => false,
                    ],
                    [
                        'type' => 'select',
                        'label' => $this->trans('Shop commission rate', [], 'Modules.Shipquik.Shipquik'),
                        'name' => '_SHIP_QUIK_shopFeeType',
                        'required' => false,
                        'options' => [
                            'query' => $this->getShopFeeType(),
                            'id' => 'id_option',
                            'name' => 'name',
                        ],
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->trans('Commission value', [], 'Modules.Shipquik.Shipquik'),
                        'name' => '_SHIP_QUIK_shopFeeValue',
                        'required' => false,
                    ],
                    [
                        'type' => 'textarea',
                        'label' => $this->trans('Shop feedback on shipping', [], 'Modules.Shipquik.Shipquik'),
                        'name' => '_SHIP_QUIK_comments',
                        'required' => false,
                    ],
                    [
                        'type' => 'select',
                        'label' => $this->trans('Print products on labels', [], 'Modules.Shipquik.Shipquik'),
                        'name' => '_SHIP_QUIK_printProducts',
                        'required' => false,
                        'options' => [
                            'query' => $this->getPrintProducts(),
                            'id' => 'id_option',
                            'name' => 'name',
                        ],
                    ],
                    [
                        'type' => 'select',
                        'label' => $this->trans('Select the change of the status of the order after being processed', [], 'Modules.Shipquik.Shipquik'),
                        'name' => '_SHIP_QUIK_orderStatus',
                        'required' => false,
                        'class' => 'subsection',
                        'options' => [
                            'query' => $this->getOrderStatus(),
                            'id' => 'id_option',
                            'name' => 'name',
                        ],
                    ],
                ],
            ],
        ];

        $form4 = [
            'form' => [
                'legend' => [
                    'title' => $this->trans('Pickup address', [], 'Modules.Shipquik.Shipquik'),
                ],
                'input' => [
                    [
                        'type' => 'text',
                        'label' => $this->trans('Company', [], 'Modules.Shipquik.Shipquik'),
                        'name' => '_SHIP_QUIK_senderAddressCompany',
                        'required' => true,
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->trans('Address', [], 'Modules.Shipquik.Shipquik'),
                        'name' => '_SHIP_QUIK_senderAddressStreet',
                        'required' => true,
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->trans('City', [], 'Modules.Shipquik.Shipquik'),
                        'name' => '_SHIP_QUIK_senderAddressCity',
                        'required' => true,
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->trans('Postal code', [], 'Modules.Shipquik.Shipquik'),
                        'name' => '_SHIP_QUIK_senderAddressPostalCode',
                        'required' => true,
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->trans('Country code', [], 'Modules.Shipquik.Shipquik'),
                        'name' => '_SHIP_QUIK_senderAddressCountryCode',
                        'required' => true,
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->trans('Phone number', [], 'Modules.Shipquik.Shipquik'),
                        'name' => '_SHIP_QUIK_senderAddressPhoneNumber',
                        'required' => true,
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->trans('Email', [], 'Modules.Shipquik.Shipquik'),
                        'name' => '_SHIP_QUIK_senderAddressEmail',
                        'required' => true,
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->trans('CIF/DNI/NIE', [], 'Modules.Shipquik.Shipquik'),
                        'name' => '_SHIP_QUIK_senderAddressCif',
                        'required' => true,
                    ],
                ],
            ],
        ];

        $form5 = [
            'form' => [
                'legend' => [
                    'title' => $this->trans('Default packages', [], 'Modules.Shipquik.Shipquik'),
                ],
                'input' => [
                    [
                        'type' => 'html',
                        'name' => '_SHIP_QUIK_shippingProductListWithoutParcelData',
                        'html_content' => $this->getProductListWithoutParcel(),
                        'required' => false,
                    ],
                    [
                        'type' => 'html',
                        'label' => $this->trans('List of packages', [], 'Modules.Shipquik.Shipquik'),
                        'name' => '_SHIP_QUIK_shippingPackageList',
                        'desc' => 'Define the packages available for your shipments',
                        'html_content' => $this->getPackageList('_SHIP_QUIK_shippingPackageList', '', Configuration::get('_SHIP_QUIK_shippingPackageList')),
                        'required' => true,
                    ],
                ],
                'submit' => [
                    'title' => $this->trans('Save', [], 'Modules.Shipquik.Shipquik'),
                    'class' => 'btn btn-default pull-right',
                ],
            ],
        ];

        $helper = new HelperForm();

        $helper->table = $this->name;
        $helper->name_controller = $submitName;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex . '&' . http_build_query(['configure' => $submitName]);
        $helper->submit_action = 'submit' . $submitName;

        $helper->default_form_language = (int) Configuration::get('PS_LANG_DEFAULT');

        if (Configuration::get('_SHIP_QUIK_suppliers')) {
            $suppliers = unserialize(Configuration::get('_SHIP_QUIK_suppliers'));
            foreach ($suppliers as $supplier) {
                $helper->fields_value['_SHIP_QUIK_suppliers_' . $supplier['name']] = true;
            }
        } else {
            $helper->fields_value['_SHIP_QUIK_suppliers'] = Tools::getValue('_SHIP_QUIK_suppliers', Configuration::get('_SHIP_QUIK_suppliers'));
        }

        $helper->fields_value['_SHIP_QUIK_pricingMode'] = Tools::getValue('_SHIP_QUIK_pricingMode', Configuration::get('_SHIP_QUIK_pricingMode'));
        $helper->fields_value['_SHIP_QUIK_chargeDUA'] = Tools::getValue('_SHIP_QUIK_chargeDUA', Configuration::get('_SHIP_QUIK_chargeDUA'));
        $helper->fields_value['_SHIP_QUIK_paymentMethod'] = Tools::getValue('_SHIP_QUIK_paymentMethod', Configuration::get('_SHIP_QUIK_paymentMethod'));
        $helper->fields_value['_SHIP_QUIK_isFreeShipping'] = Tools::getValue('_SHIP_QUIK_isFreeShipping', Configuration::get('_SHIP_QUIK_isFreeShipping'));
        $helper->fields_value['_SHIP_QUIK_freeShippingPriceFrom'] = Tools::getValue('_SHIP_QUIK_freeShippingPriceFrom', Configuration::get('_SHIP_QUIK_freeShippingPriceFrom') ? Configuration::get('_SHIP_QUIK_freeShippingPriceFrom') : '0');
        $helper->fields_value['_SHIP_QUIK_shopFeeType'] = Tools::getValue('_SHIP_QUIK_shopFeeType', Configuration::get('_SHIP_QUIK_shopFeeType'));
        $helper->fields_value['_SHIP_QUIK_shopFeeValue'] = Tools::getValue('_SHIP_QUIK_shopFeeValue', Configuration::get('_SHIP_QUIK_shopFeeValue') ? Configuration::get('_SHIP_QUIK_shopFeeValue') : 0);
        $helper->fields_value['_SHIP_QUIK_comments'] = Tools::getValue('_SHIP_QUIK_comments', Configuration::get('_SHIP_QUIK_comments') ? Configuration::get('_SHIP_QUIK_comments') : '');
        $helper->fields_value['_SHIP_QUIK_printProducts'] = Tools::getValue('_SHIP_QUIK_printProducts', Configuration::get('_SHIP_QUIK_printProducts') ? Configuration::get('_SHIP_QUIK_printProducts') : '0');
        $helper->fields_value['_SHIP_QUIK_orderStatus'] = Tools::getValue('_SHIP_QUIK_orderStatus', Configuration::get('_SHIP_QUIK_orderStatus') ? Configuration::get('_SHIP_QUIK_orderStatus') : '0');

        $helper->fields_value['_SHIP_QUIK_pickUpTimeText'] = Tools::getValue('_SHIP_QUIK_pickUpTimeText', Configuration::get('_SHIP_QUIK_pickUpTimeText') ? Configuration::get('_SHIP_QUIK_pickUpTimeText') : '');

        $helper->fields_value['_SHIP_QUIK_senderAddressCompany'] = Tools::getValue('_SHIP_QUIK_senderAddressCompany', Configuration::get('_SHIP_QUIK_senderAddressCompany'));
        $helper->fields_value['_SHIP_QUIK_senderAddressStreet'] = Tools::getValue('_SHIP_QUIK_senderAddressStreet', Configuration::get('_SHIP_QUIK_senderAddressStreet'));
        $helper->fields_value['_SHIP_QUIK_senderAddressCity'] = Tools::getValue('_SHIP_QUIK_senderAddressCity', Configuration::get('_SHIP_QUIK_senderAddressCity'));
        $helper->fields_value['_SHIP_QUIK_senderAddressPostalCode'] = Tools::getValue('_SHIP_QUIK_senderAddressPostalCode', Configuration::get('_SHIP_QUIK_senderAddressPostalCode'));
        $helper->fields_value['_SHIP_QUIK_senderAddressCountryCode'] = Tools::getValue('_SHIP_QUIK_senderAddressCountryCode', Configuration::get('_SHIP_QUIK_senderAddressCountryCode') ? Configuration::get('_SHIP_QUIK_senderAddressCountryCode') : 'ES');
        $helper->fields_value['_SHIP_QUIK_senderAddressPhoneNumber'] = Tools::getValue('_SHIP_QUIK_senderAddressPhoneNumber', Configuration::get('_SHIP_QUIK_senderAddressPhoneNumber'));
        $helper->fields_value['_SHIP_QUIK_senderAddressEmail'] = Tools::getValue('_SHIP_QUIK_senderAddressEmail', Configuration::get('_SHIP_QUIK_senderAddressEmail'));
        $helper->fields_value['_SHIP_QUIK_senderAddressCif'] = Tools::getValue('_SHIP_QUIK_senderAddressCif', Configuration::get('_SHIP_QUIK_senderAddressCif'));

        return $helper->generateForm([$form1, $form2, $form3, $form4, $form5]);
    }

    private function checkRequiredFields()
    {
        $required_fields = [
            'suppliers',
            'pricingMode',
            'chargeDUA',
            'pickUpTimeText',
            'senderAddressCompany',
            'senderAddressStreet',
            'senderAddressCity',
            'senderAddressPostalCode',
            'senderAddressCountryCode',
            'senderAddressPhoneNumber',
            'senderAddressEmail',
            'senderAddressCif',
            'shippingPackageList',
        ];

        foreach ($required_fields as $field) {
            $value = $this->config->getValueByKey($field);
            if (!$value) {
                return true;
            }
        }

        return false;
    }

    /* Carrier & Shipping */

    public function hookActionCarrierUpdate($params)
    {
        $controller = $this->getHookController('actionCarrierUpdate');

        return $controller->run($params);
    }

    public function hookDisplayCarrierExtraContent($params)
    {
        $controller = $this->getHookController('displayCarrierExtraContent');

        return $controller->run($params);
    }

    public function getOrderShippingCost($params, $shipping_cost)
    {
        $controller = $this->getHookController('getOrderShippingCost');

        return $controller->run($params, $shipping_cost);
    }

    public function getOrderShippingCostExternal($params)
    {
        return $this->getOrderShippingCost($params, 0);
    }

    /* Orders */

    public function hookDisplayOrderConfirmation($params)
    {
        $controller = $this->getHookController('displayOrderConfirmation');

        return $controller->run($params);
    }

    public function hookDisplaydisplayAdminGridTableAfter($params)
    {
        $controller = $this->getHookController('displayAdminGridTableAfter');

        return $controller->run($params);
    }

    public function hookActionOrderGridDefinitionModifier($params)
    {
        $controller = $this->getHookController('actionOrderGridDefinitionModifier');

        return $controller->run($params);
    }

    public function hookActionOrderGridDataModifier($params)
    {
        $controller = $this->getHookController('actionOrderGridDataModifier');

        return $controller->run($params);
    }

    public function hookActionOrderGridQueryBuilderModifier($params)
    {
        $controller = $this->getHookController('actionOrderGridQueryBuilderModifier');

        return $controller->run($params);
    }

    public function hookActionAdminOrdersListingFieldsModifier($params)
    {
        $controller = $this->getHookController('actionAdminOrdersListingFieldsModifier');

        return $controller->run($params);
    }

    public function hookActionAdminOrdersListingResultsModifier($params)
    {
        $controller = $this->getHookController('actionAdminOrdersListingResultsModifier');

        return $controller->run($params);
    }

    // TODO: No se usa, es para versiones menores que la 1.7.7.0
    public function hookActionOrderGridPresenterModifier($params)
    {
        $controller = $this->getHookController('actionOrderGridPresenterModifier');

        return $controller->run($params);
    }

    public function hookActionAdminControllerSetMedia($params)
    {
        $controller = $this->getHookController('actionAdminControllerSetMedia');

        return $controller->run($params);
    }

    public function getActivationFormFields()
    {
        return [
            '_SHIP_QUIK_activationKey' => ['required' => true, 'type' => 'text'],
            '_SHIP_QUIK_activationEmail' => ['required' => true, 'type' => 'text'],
        ];
    }

    public function getFormFields()
    {
        return [
            '_SHIP_QUIK_suppliers' => ['required' => true, 'type' => 'serialized'],
            '_SHIP_QUIK_pricingMode' => ['required' => true, 'type' => 'select'],
            '_SHIP_QUIK_chargeDUA' => ['required' => true, 'type' => 'select'],
            '_SHIP_QUIK_paymentMethod' => ['required' => false, 'type' => 'select'],
            '_SHIP_QUIK_isFreeShipping' => ['required' => false, 'type' => 'select'],
            '_SHIP_QUIK_shippingTableRates' => ['required' => false, 'type' => 'html'],
            '_SHIP_QUIK_freeShippingPriceFrom' => ['required' => false, 'type' => 'number'],
            '_SHIP_QUIK_shopFeeType' => ['required' => false, 'type' => 'select'],
            '_SHIP_QUIK_shopFeeValue' => ['required' => false, 'type' => 'number'],
            '_SHIP_QUIK_comments' => ['required' => false, 'type' => 'text'],
            '_SHIP_QUIK_printProducts' => ['required' => false, 'type' => 'text'],
            '_SHIP_QUIK_orderStatus' => ['required' => false, 'type' => 'text'],
            '_SHIP_QUIK_pickUpTimeText' => ['required' => true, 'type' => 'text'],
            '_SHIP_QUIK_senderAddressCompany' => ['required' => true, 'type' => 'text'],
            '_SHIP_QUIK_senderAddressStreet' => ['required' => true, 'type' => 'text'],
            '_SHIP_QUIK_senderAddressCity' => ['required' => true, 'type' => 'text'],
            '_SHIP_QUIK_senderAddressPostalCode' => ['required' => true, 'type' => 'text'],
            '_SHIP_QUIK_senderAddressCountryCode' => ['required' => true, 'type' => 'text'],
            '_SHIP_QUIK_senderAddressPhoneNumber' => ['required' => true, 'type' => 'text'],
            '_SHIP_QUIK_senderAddressEmail' => ['required' => true, 'type' => 'text'],
            '_SHIP_QUIK_senderAddressCif' => ['required' => true, 'type' => 'text'],
            '_SHIP_QUIK_shippingPackageList' => ['required' => true, 'type' => 'html'],
        ];
    }
}
