<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/OSL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author Ship-Quik SL <info@ship-quik.com>
 * @copyright Since 2022 Ship-Quik SL
 * @license   https://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 */

namespace ShipQuik\PrestaShop;

use ShipQuik\Shared\ShipQuik_Helper;

class ShipQuik_BD
{
    public function __construct()
    {
    }

    public static function getDatabyOrder($order)
    {
        $where = '`id_cart` = ' . (int) $order->id_cart;
        $where .= ' AND `id_shop_group` = ' . (int) $order->id_shop_group;
        $where .= ' AND `id_shop` = ' . (int) $order->id_shop;
        $where .= ' AND `id_carrier` = ' . (int) $order->id_carrier;
        $where .= ' AND `id_order` = ' . (int) $order->id;

        $query = new \DbQuery();
        $query->select('*')
            ->from(bqSQL('shipquik_cart'))
            ->where($where);

        $result = \Db::getInstance()->executeS($query);

        if (!empty($result)) {
            $result[0]['supplier_data'] = ShipQuik_Helper::mb_unserialize(base64_decode($result[0]['supplier_data']));

            return $result[0];
        }

        return false;
    }

    public static function getDatabyReference($reference)
    {
        $where = '`shipment_data` LIKE "%' . $reference . '%"';

        $query = new \DbQuery();
        $query->select('*')
            ->from(bqSQL('shipquik_cart'))
            ->where($where);

        $result = \Db::getInstance()->executeS($query);

        if (!empty($result)) {
            $result[0]['supplier_data'] = ShipQuik_Helper::mb_unserialize(base64_decode($result[0]['supplier_data']));

            return $result[0];
        }

        return false;
    }

    public static function getDatabyOrderId($order_id)
    {
        $where = '`id_order` = ' . (int) $order_id;

        $query = new \DbQuery();
        $query->select('*')
            ->from(bqSQL('shipquik_cart'))
            ->where($where);

        $result = \Db::getInstance()->executeS($query);

        return empty($result) ? false : $result[0];
    }

    public static function getDatabyCart($cart, $id_carrier)
    {
        $where = '`id_cart` = ' . (int) $cart->id;
        $where .= ' AND `id_shop_group` = ' . (int) $cart->id_shop_group;
        $where .= ' AND `id_shop` = ' . (int) $cart->id_shop;
        $where .= ' AND `id_carrier` = ' . (int) $id_carrier;

        $query = new \DbQuery();
        $query->select('supplier_data')
            ->from(bqSQL('shipquik_cart'))
            ->where($where);

        $result = \Db::getInstance()->executeS($query);

        return empty($result) || empty($result[0]['supplier_data']) ? false : ShipQuik_Helper::mb_unserialize(base64_decode($result[0]['supplier_data']));
    }

    public static function getShipmentDatabyCart($cart, $id_carrier)
    {
        $where = '`id_cart` = ' . (int) $cart->id;
        $where .= ' AND `id_shop_group` = ' . (int) $cart->id_shop_group;
        $where .= ' AND `id_shop` = ' . (int) $cart->id_shop;
        $where .= ' AND `id_carrier` = ' . (int) $id_carrier;

        $query = new \DbQuery();
        $query->select('shipment_data')
            ->from(bqSQL('shipquik_cart'))
            ->where($where);

        $result = \Db::getInstance()->executeS($query);

        return empty($result) || empty($result[0]['shipment_data']) ? false : json_decode($result[0]['shipment_data'], JSON_UNESCAPED_UNICODE);
    }

    public static function getTotalPaymentbyCart($cart, $id_carrier)
    {
        $where = '`id_cart` = ' . (int) $cart->id;
        $where .= ' AND `id_shop_group` = ' . (int) $cart->id_shop_group;
        $where .= ' AND `id_shop` = ' . (int) $cart->id_shop;
        $where .= ' AND `id_carrier` = ' . (int) $id_carrier;

        $query = new \DbQuery();
        $query->select('total_payment')
            ->from(bqSQL('shipquik_cart'))
            ->where($where);

        $result = \Db::getInstance()->executeS($query);

        return empty($result) || empty($result[0]['total_payment']) ? false : json_decode($result[0]['total_payment'], JSON_UNESCAPED_UNICODE);
    }

    public static function setOrderIdbyOrder($order)
    {
        $where = '`id_cart` = ' . (int) $order->id_cart;
        $where .= ' AND `id_shop_group` = ' . (int) $order->id_shop_group;
        $where .= ' AND `id_shop` = ' . (int) $order->id_shop;
        $where .= ' AND `id_carrier` = ' . (int) $order->id_carrier;

        $result = \Db::getInstance()->update('shipquik_cart', [
            'id_order' => (int) $order->id,
            'date_updated' => date('Y-m-d H:i:s'),
        ], $where, 1, true);

        return $result;
    }

    public static function setTotalPayment($order, $totalPayment)
    {
        $where = '`id_order` = ' . (int) $order->id;
        $where .= ' AND `id_cart` = ' . (int) $order->id_cart;
        $where .= ' AND `id_shop_group` = ' . (int) $order->id_shop_group;
        $where .= ' AND `id_shop` = ' . (int) $order->id_shop;
        $where .= ' AND `id_carrier` = ' . (int) $order->id_carrier;

        $result = \Db::getInstance()->update('shipquik_cart', [
            'total_payment' => $totalPayment,
            'date_updated' => date('Y-m-d H:i:s'),
        ], $where, 1, true);

        return $result;
    }

    public static function setPickupDatetime($order, $pickupDatetime)
    {
        $where = '`id_order` = ' . (int) $order->id;
        $where .= ' AND `id_cart` = ' . (int) $order->id_cart;
        $where .= ' AND `id_shop_group` = ' . (int) $order->id_shop_group;
        $where .= ' AND `id_shop` = ' . (int) $order->id_shop;
        $where .= ' AND `id_carrier` = ' . (int) $order->id_carrier;

        $result = \Db::getInstance()->update('shipquik_cart', [
            'pickup_datetime' => json_encode($pickupDatetime),
            'date_updated' => date('Y-m-d H:i:s'),
        ], $where, 1, true);

        return $result;
    }

    public static function setPaidStatus($order, $status)
    {
        $where = '`id_order` = ' . (int) $order->id;
        $where .= ' AND `id_cart` = ' . (int) $order->id_cart;
        $where .= ' AND `id_shop_group` = ' . (int) $order->id_shop_group;
        $where .= ' AND `id_shop` = ' . (int) $order->id_shop;
        $where .= ' AND `id_carrier` = ' . (int) $order->id_carrier;

        $result = \Db::getInstance()->update('shipquik_cart', [
            'paid' => $status,
            'date_updated' => date('Y-m-d H:i:s'),
        ], $where, 1, true);

        return $result;
    }

    public static function setPrintedStatus($order, $status)
    {
        $where = '`id_order` = ' . (int) $order->id;
        $where .= ' AND `id_cart` = ' . (int) $order->id_cart;
        $where .= ' AND `id_shop_group` = ' . (int) $order->id_shop_group;
        $where .= ' AND `id_shop` = ' . (int) $order->id_shop;
        $where .= ' AND `id_carrier` = ' . (int) $order->id_carrier;

        $result = \Db::getInstance()->update('shipquik_cart', [
            'printed' => $status,
            'date_updated' => date('Y-m-d H:i:s'),
        ], $where, 1, true);

        return $result;
    }

    public static function setShipmentDatabyOrder($order, $data)
    {
        $where = '`id_cart` = ' . (int) $order->id_cart;
        $where .= ' AND `id_shop_group` = ' . (int) $order->id_shop_group;
        $where .= ' AND `id_shop` = ' . (int) $order->id_shop;
        $where .= ' AND `id_carrier` = ' . (int) $order->id_carrier;
        $where .= ' AND `id_order` = ' . (int) $order->id;

        $result = \Db::getInstance()->update('shipquik_cart', [
            'shipment_data' => json_encode($data, JSON_UNESCAPED_UNICODE),
            'date_updated' => date('Y-m-d H:i:s'),
        ], $where, 1, true);

        return $result;
    }

    public static function setShipmentData($cart, $id_carrier, $data)
    {
        $where = '`id_cart` = ' . (int) $cart->id;
        $where .= ' AND `id_shop_group` = ' . (int) $cart->id_shop_group;
        $where .= ' AND `id_shop` = ' . (int) $cart->id_shop;
        $where .= ' AND `id_carrier` = ' . (int) $id_carrier;

        $query = new \DbQuery();
        $query->select('id_cart')
            ->from(bqSQL('shipquik_cart'))
            ->where($where);

        $result = \Db::getInstance()->executeS($query);

        if (empty($result)) {
            $sqlResult = \Db::getInstance()->insert(
                'shipquik_cart',
                [
                    'id_cart' => (int) $cart->id,
                    'id_shop_group' => (int) $cart->id_shop_group,
                    'id_shop' => (int) $cart->id_shop,
                    'id_carrier' => (int) $id_carrier,
                    'supplier_data' => base64_encode(serialize($data)),
                    'date_updated' => date('Y-m-d H:i:s'),
                ]
            );
            $pepe = '';
        } else {
            $sqlResult3 = \Db::getInstance()->update('shipquik_cart', [
                'supplier_data' => base64_encode(serialize($data)),
                'date_updated' => date('Y-m-d H:i:s'),
            ], $where, 1, true);
        }
    }

    public static function setCacheData($field, $cart, $id_carrier, $data)
    {
        if (is_null($cart->id)) {
            return false;
        }

        $where = '`id_cart` = ' . (int) $cart->id;
        $where .= ' AND `id_shop_group` = ' . (int) $cart->id_shop_group;
        $where .= ' AND `id_shop` = ' . (int) $cart->id_shop;
        $where .= ' AND `id_carrier` = ' . (int) $id_carrier;

        $query = new \DbQuery();
        $query->select('id_cart')
            ->from(bqSQL('shipquik_cart'))
            ->where($where);

        $result = \Db::getInstance()->executeS($query);

        if (empty($result)) {
            $result = \Db::getInstance()->insert(
                'shipquik_cart',
                [
                    'id_cart' => (int) $cart->id,
                    'id_shop_group' => (int) $cart->id_shop_group,
                    'id_shop' => (int) $cart->id_shop,
                    'id_carrier' => (int) $id_carrier,
                    $field => base64_encode(serialize($data)),
                    'date_updated' => date('Y-m-d H:i:s'),
                ]
            );
        } else {
            $result = \Db::getInstance()->update('shipquik_cart', [
                $field => base64_encode(serialize($data)),
                'date_updated' => date('Y-m-d H:i:s'),
            ], $where, 1, true);
        }
    }

    public static function getCacheData($field, $cart, $id_carrier)
    {
        if (is_null($cart->id)) {
            return false;
        }

        $where = '`id_cart` = ' . (int) $cart->id;
        $where .= ' AND `id_shop_group` = ' . (int) $cart->id_shop_group;
        $where .= ' AND `id_shop` = ' . (int) $cart->id_shop;
        $where .= ' AND `id_carrier` = ' . (int) $id_carrier;

        $query = new \DbQuery();
        $query->select($field)
            ->from(bqSQL('shipquik_cart'))
            ->where($where);

        $result = \Db::getInstance()->executeS($query);

        return empty($result) || empty($result[0][$field]) ? false : ShipQuik_Helper::mb_unserialize(base64_decode($result[0][$field]));
    }
}
