<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/OSL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author Ship-Quik SL <info@ship-quik.com>
 * @copyright Since 2022 Ship-Quik SL
 * @license   https://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 */

namespace ShipQuik\PrestaShop;

use Cart;

class ShipQuik_ShippingRate
{
    protected $parent;

    protected $config;

    protected $cart;

    public $rate;

    public function __construct($parent, $rate, $cart)
    {
        $this->parent = $parent;
        $this->rate = $rate;

        $this->cart = $cart;

        $this->config = $parent->config;
    }

    /**
     * @return array
     */
    public function getRate($cart, $pricingMode, $shippingPackage)
    {
        if (!$pricingMode) {
            return false;
        }

        $changeDUA = $this->config->getValueByKey('chargeDUA');

        $result = $pricingMode->getModeRate($this);

        if ($result) {
            $DUACost = ($changeDUA ? $result['dua'] : 0);

            $isFree = $this->isFreeShipping($this->rate);

            // If is freeShipping, not continue
            if ($isFree || $this->hasShopFreeFrom()) {
                $sql = ShipQuik_BD::setShipmentData($cart, $this->parent->id_carrier, $result);

                return 0 + $DUACost;
            }

            // If has tableRate
            $tableRatePrice = $this->getShipQuikTableRate();
            if ($tableRatePrice > 0) {
                $sql = ShipQuik_BD::setShipmentData($cart, $this->parent->id_carrier, $result);

                return $tableRatePrice + $DUACost;
            }

            $cost = $this->getShopFee($result['salesPrice']) + $DUACost;

            $sql = ShipQuik_BD::setShipmentData($cart, $this->parent->id_carrier, $result);

            return $cost + $DUACost;
        }

        return false;
    }

    public function getSupplier($rate)
    {
        $shippingSupplierService = explode('_', $rate->get_method_id());

        if (isset($shippingSupplierService[1])) {
            return $shippingSupplierService[1];
        }

        return false;
    }

    public function getService($rate)
    {
        $shippingSupplierService = explode('-', $rate->get_method_id());

        if (isset($shippingSupplierService[2])) {
            return strtoupper($shippingSupplierService[2]);
        }

        return false;
    }

    private function isFreeShipping($rate)
    {
        if ($this->config->getValueByKey('isFreeShipping')) {
            return true;
        }

        return false;
    }

    private function hasShopFreeFrom()
    {
        $total = $this->getCartSubTotal($this->cart);

        if ((float) $this->config->getValueByKey('freeShippingPriceFrom') > 0 && $total >= (float) $this->config->getValueByKey('freeShippingPriceFrom')) {
            return true;
        }

        return false;
    }

    private function getShopFee($price)
    {
        if ($this->hasShopFreeFrom()) {
            return $price;
        }

        $shopFeeValue = (float) $this->config->getValueByKey('shopFeeValue');
        $shopFeeType = $this->config->getValueByKey('shopFeeType');

        $price = (float) $price;

        if ($shopFeeValue > 0) {
            if ($shopFeeType == 'price') {
                $price = $price + $shopFeeValue;
            } else {
                $price = $price + ($price * $shopFeeValue / 100);
            }
        }

        return $price;
    }

    private function getShipQuikTableRate()
    {
        $resultRates = 0;

        $total = $this->getCartSubTotal($this->cart);

        $tableRates = $this->config->getValueByKey('shippingTableRates');

        if (!$tableRates) {
            $tableRates = [];
        } else {
            $tableRates = unserialize($tableRates);
        }

        if (!empty($tableRates)) {
            foreach ($tableRates as $rate) {
                if ($total >= (int) $rate['from'] && $total <= (int) $rate['to']) {
                    $resultRates = (float) $rate['value'];
                }
            }
        }

        return $resultRates;
    }

     // $this->cart->getTotalCart(true, Cart::BOTH);
    // \Cart::getOrderTotalUsingTaxCalculationMethod($this->cart->id);
    private function getCartSubTotal($cart)
    {
        $subtotal = 0;

        foreach ($cart->getProducts() as $product) {
            $subtotal = $subtotal + $product['total_wt'];
        }

        return $subtotal;
    }
}
