<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/OSL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author Ship-Quik SL <info@ship-quik.com>
 * @copyright Since 2022 Ship-Quik SL
 * @license   https://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 */

namespace ShipQuik\Shared;

class ShipQuik_Auth
{
    private static $_instance = null;

    protected $parent;

    protected $config;

    protected $data;

    public function __construct($parent = null)
    {
        $this->parent = $parent;
        $this->config = $parent->config;

        $this->data = $this->getData();
    }

    private function getData()
    {
        $auth = [
            'email' => $this->config->getValueByKey('activationEmail'),
            'securityKey' => $this->config->getValueByKey('activationKey'),
        ];

        return $auth;
    }

    public function send()
    {
        $params = [];
        $params['Baseurl'] = $this->config->getValueByKey('endpoints')['auth'];
        $params['Data'] = $this->data;

        $curl = new ShipQuik_API($this->parent, $params, false);

        $auth = $curl->runCall();

        if ($auth) {
            $decoded = $this->config->getTokenDecoded($auth);
            $decoded->token = $auth;

            $this->config->update_option($this->config->base . 'paymentMethod', $decoded->paymentMethod);
            $this->config->setAccessToken($decoded);

            return true;
        }

        return false;
    }

    public static function instance($parent)
    {
        if (is_null(self::$_instance)) {
            self::$_instance = new self($parent);
        }

        return self::$_instance;
    } // End instance()

    public function __clone()
    {
        trigger_error('La clonación de este objeto no está permitida', E_USER_ERROR);
    }
}
