<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/OSL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author Ship-Quik SL <info@ship-quik.com>
 * @copyright Since 2022 Ship-Quik SL
 * @license   https://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 */

namespace ShipQuik\Shared;

use Ramsey\Uuid\Uuid;

class ShipQuik_Helper
{
    private static $_instance = null;

    protected $parent = null;

    protected $config = null;

    public function __construct($parent)
    {
        $this->parent = $parent;
        $this->config = $parent->config;
    }

    public static function searchArrayValueByKey($search, array $array)
    {
        foreach (new \RecursiveIteratorIterator(new \RecursiveArrayIterator($array)) as $key => $value) {
            if ($search === $key) {
                return $value;
            }
        }

        return false;
    }

    public static function deleteShippingMethodFiles($files)
    {
        $files = glob("$files", GLOB_BRACE);

        if (count($files) > 0) {
            foreach ($files as $file) {
                unlink($file);
            }
        }
    }

    public static function getDateTime($date, $time)
    {
        return new \DateTime($date . ' ' . $time);
    }

    public static function getIfTaricNeededES($parent, $data)
    {
        $result = false;

        $params = [];

        $params['Baseurl'] = $parent->getValueByKey('endpoints')['getIfTaricNeededES'];
        $params['Data'] = $data;

        // d(json_encode($params['Data']));

        $curl = new ShipQuik_API($parent, $params, true);
        $result = $curl->runCall();

        if ($result == 'renew_token') {
            $parent->config->reNewAccessToken();

            $result = $curl->runCall();
        }

        if ($result) {
            if (isset($result['data'])) {
                return $result;
            }
        }

        return false;
    }

    public static function createStripeIntent($parent, $data)
    {
        $result = false;

        $params = [];

        $params['Baseurl'] = $parent->config->getValueByKey('endpoints')['createStripeIntent'];
        $params['Data'] = $data;

        $curl = new ShipQuik_API($parent, $params, true);
        $result = $curl->runCall();

        if ($result == 'renew_token') {
            $parent->config->reNewAccessToken();

            $result = $curl->runCall();
        }

        if ($result) {
            return $result;
        }

        return false;
    }

    public static function getSuppliersConfiguration($parent, $data)
    {
        $result = false;

        $params = [];

        $params['Baseurl'] = $parent->config->getValueByKey('endpoints')['getSuppliersConfiguration'];
        $params['Data'] = $data;

        $curl = new ShipQuik_API($parent, $params, false);
        $result = $curl->runCall();

        if ($result == 'renew_token') {
            $parent->config->reNewAccessToken();

            $result = $curl->runCall();
        }

        if ($result) {
            return $result;
        }

        return false;
    }

    public static function mb_unserialize($string)
    {
        $string2 = preg_replace_callback(
            '!s:(\d+):"(.*?)";!s',
            function ($element) {
                $len = strlen($element[2]);
                $result = "s:$len:\"{$element[2]}\";";

                return $result;
            },
            $string
        );

        return unserialize($string2);
    }

    public static function getUUID($supplierCode)
    {
        $lenght = 16;

        if ($supplierCode == 'MONDIAL_RELAY') {
            $lenght = 9;
        }
        if ($supplierCode == 'MRW') {
            $lenght = 12;
        }

        $uuid = str_replace('-', '', Uuid::uuid4());

        return substr($uuid, 0, $lenght);
    }

    public static function encodeKey($data)
    {
        $result = base64_encode(json_encode($data));

        return $result;
    }

    public static function decodeKey($data)
    {
        $result = json_decode(base64_decode($data));

        return $result;
    }

    public static function instance($parent)
    {
        if (is_null(self::$_instance)) {
            self::$_instance = new self($parent);
        }

        return self::$_instance;
    } // End instance()

    public function __clone()
    {
        trigger_error('La clonación de este objeto no está permitida', E_USER_ERROR);
    }
}
