<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/OSL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author Ship-Quik SL <info@ship-quik.com>
 * @copyright Since 2022 Ship-Quik SL
 * @license   https://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 */

namespace ShipQuik\Shared;

class ShipQuik_PricingMode
{
    private static $_instance = null;

    protected $parent;

    protected $config;

    public $pricingMode;

    public $services;

    public $packages;

    public $shiQuikSuppliers;

    public function __construct($parent = null, $pricingMode = 'price')
    {
        $this->parent = $parent;
        $this->pricingMode = $pricingMode;

        $this->config = $parent->config;

        $this->services = $this->getModeConfig();
    }

    public static function create($parent, $pricingMode = 'price')
    {
        return new self($parent, $pricingMode);
    }

    public function getForRates($shiQuikSuppliers)
    {
        $this->shiQuikSuppliers = $shiQuikSuppliers;
        $this->packages = $shiQuikSuppliers->getSuppliersResult($this->isServicesListMode(), $this->pricingMode);

        return $this;
    }

     private function isServicesListMode()
     {
         if ($this->pricingMode == 'serviceslist') {
             return $this->getSuppliersId($this->services['servicesList']);
         }

         return $this->config->getValueByKey('suppliers');
     }

    private function getSuppliersId()
    {
        $result = [];

        foreach ($this->services['servicesList'] as $service) {
            $result[] = $service['supplierId'];
        }

        return array_unique($result);
    }

    private function getModeConfig()
    {
        // PricingMode values
        // speed:        Precio del más rapido (uno).
        // price:        Precio del mas barato (uno).
        // serviceslist: Lista de servicios (needs servicesList object) (seleccción).
        // speedprice:   Puede eligir mas rapido o mas barato (seleccción).

        $speedName = $this->parent->getTranslator()->trans('by Ship-Quik fast shipping', [], 'Modules.Shipquik.Shipquik');
        $priceName = $this->parent->getTranslator()->trans('by Ship-Quik low cost', [], 'Modules.Shipquik.Shipquik');

        switch ($this->pricingMode) {
            case 'speed':
                return [
                    'cost' => 0,
                    'servicesList' => [
                        [
                            'enabled' => true,
                            'supplierId' => 'speed',
                            'serviceId' => 'speed',
                            'name' => $speedName,
                            'description' => $speedName,
                            'countries' => ['ALL'],
                        ],
                    ],
                ];
            case 'price':
                return [
                    'cost' => 0,
                    'servicesList' => [
                        [
                            'enabled' => true,
                            'supplierId' => 'price',
                            'serviceId' => 'price',
                            'name' => $priceName,
                            'description' => $priceName,
                            'countries' => ['ALL'],
                        ],
                    ],
                ];
            case 'serviceslist':
                return [
                    'cost' => 0,
                    'servicesList' => $this->config->getValueByKey('servicesList'),
                ];
            case 'speedprice':
                return [
                    'cost' => 0,
                    'servicesList' => [
                        [
                            'enabled' => true,
                            'supplierId' => 'price',
                            'serviceId' => 'price',
                            'name' => $speedName,
                            'description' => $speedName,
                            'countries' => ['ALL'],
                        ],
                        [
                            'enabled' => true,
                            'supplierId' => 'speed',
                            'serviceId' => 'speed',
                            'name' => $priceName,
                            'description' => $priceName,
                            'countries' => ['ALL'],
                        ],
                    ],
                ];
        }
    }

    public function getModeRate($rateObject = null)
    {
        // PricingMode values
        // speed:        Precio del más rapido (uno).
        // price:        Precio del mas barato (uno).
        // serviceslist: Lista de servicios (needs servicesList object) (seleccción).
        // speedprice:   Puede eligir mas rapido o mas barato (seleccción).

        if (sizeof($this->packages) > 0) {
            switch ($this->pricingMode) {
                case 'speed':
                    return $this->packages[0];
                case 'price':
                    return $this->packages[0];
                case 'serviceslist':
                    $name = $rateObject->getService($rateObject->rate);

                    return $this->shiQuikSuppliers->getServiceByName($this->packages, $name);
                case 'speedprice':
                    if ($rateObject->rate == 'speed') {
                        return $this->shiQuikSuppliers->getServiceByType($this->packages, 'speed');
                    } else {
                        return $this->shiQuikSuppliers->getServiceByType($this->packages, 'price');
                    }
            }
        }

        return [];
    }

    public static function instance()
    {
        if (!isset(self::$_instance)) {
            $class = __CLASS__;
            self::$_instance = new $class();
        }

        return self::$_instance;
    }

    public function __clone()
    {
        trigger_error('La clonación de este objeto no está permitida', E_USER_ERROR);
    }
}
